INSTALL_LOG="/var/log/ufm_ha_install.log"
UFM_WATCHER_FILE="${SCRIPT_DIR}/scripts/ufm_ha_watcher"
UFM_HA_CONFIG="${SCRIPT_DIR}/scripts/ha_utils/ufm-ha.conf"

# UFM Product Names
PRODUCT_CYBER_AI="cyber-ai"
PRODUCT_ENTERPRISE="enterprise"
PRODUCT_TELEMETRY="telemetry"
PRODUCT_UFM_CYBER_AI="ufm-cyber-ai"

bin_scripts=("ufm_ha_cluster" "ufm_ha_watcher" "configure_ha_nodes.sh" "telemetry_watcher" "replace_ha_nodes.sh")
services=("ufm-ha-watcher.service" "telemetry-watcher.service")
subdirs=("ha_utils")

LOCAL_BIN_DIR="/usr/local/bin"
SERVICES_DIR="/etc/systemd/system"
UFM_HA_INSTALL_LOC="/opt/ufm_ha"

DRBD_LOCATION=
DRBD_DISK=
PRODUCT=

cleanup_install() {
    popd > /dev/null
}

log_msg(){
    echo "$@" | tee -a ${INSTALL_LOG}
}

_copy_services() {
    for service_name in "${services[@]}"; do
        \cp -f ${service_name} ${SERVICES_DIR}
    done
}

_copy_binaries() {
    for bin_script in "${bin_scripts[@]}"; do
        \cp -f ${bin_script} ${LOCAL_BIN_DIR}
        chmod +x ${LOCAL_BIN_DIR}/${bin_script}
    done
}

_copy_subdirs() {
    for subdir in "${subdirs[@]}"; do
        \cp -rf ${subdir} ${LOCAL_BIN_DIR}
    done    
}

_rm_services() {
    for service_name in "${services[@]}"; do
        \rm -f ${SERVICES_DIR}/${service_name}
    done
}

_rm_binaries() {
    for bin_script in "${bin_scripts[@]}"; do
        \rm -f  ${LOCAL_BIN_DIR}/${bin_script}
    done
}

_rm_subdirs() {
    for subdir in "${subdirs[@]}"; do
        \rm -rf ${LOCAL_BIN_DIR}/${subdir}
    done    
}

check_pre_reqs(){
    log_msg " -I- Checking required packages..."
    if [[ -f /etc/os-release ]]; then
        # Import os-release file
        . /etc/os-release

        # Check OS ID
        if [[ ${ID} == "ubuntu" ]]; then
            check_cmd="dpkg -l 2>/dev/null | grep -q "
            req_packages=("pcs" "drbd-utils" "resource-agents" "bind9-host" "pacemaker")

        elif [[ ${ID} == "rhel" || ${ID} == "centos" ]]; then
            check_cmd="rpm -qa | grep -q "
            req_packages=("pcs" "drbd" "kmod-drbd" "bind-utils" "pacemaker")
        else
            log_msg "WARNING: Unsupported OS"
            return 1
        fi

        # For Telemetry we need to make sure curl is installed because
        # we use it in the telemetry-watcher logic.
        if [[ ${PRODUCT} == ${PRODUCT_TELEMETRY} ]]; then
            req_packages+=("curl")
        fi

        errors=0
        for pkg in "${req_packages[@]}"; do
            # Checking required packages
            eval "${check_cmd} ${pkg}"
            [ $? -ne 0 ] && log_msg "- ERROR - Missing required package: ${pkg}" && errors=1
        done

        if [[ ${errors} -eq 1 ]]; then
            log_msg "- ERROR - Please install missing packages and try again."
            return 1
        fi

    else
        log_msg "- ERROR - Can't read OS release file"
        return 1
    fi

    log_msg " -I- Done checking required packages."
    return 0
}

copy_files() {
    log_msg " -I- Copying HA services to ${SERVICES_DIR}..."
    _copy_services
    log_msg " -I- Copying HA scripts to ${LOCAL_BIN_DIR}..."
    _copy_binaries
    _copy_subdirs
    log_msg " -I- Creating dir ${UFM_HA_INSTALL_LOC} and copying uninstall files..."
    mkdir -p ${UFM_HA_INSTALL_LOC}
    \cp ../.install_common.sh ${UFM_HA_INSTALL_LOC}
    \cp ../uninstall_ha.sh ${UFM_HA_INSTALL_LOC}
    log_msg " -I- Done Copying!"
}

rm_files() {
    echo "Removing ha services from ${SERVICES_DIR}..."
    _rm_services
    echo "Removing ha scripts from ${LOCAL_BIN_DIR}..."
    _rm_binaries
    _rm_subdirs
    echo "Removing dir ${UFM_HA_INSTALL_LOC} and its content..."
    \rm -rf ${UFM_HA_INSTALL_LOC}
}


disable_pcs_gui() {
    local pcsd_config="/etc/default/pcsd"
    if [ -f ${pcsd_config} ]; then
        gui_attr=$(grep PCSD_DISABLE_GUI ${pcsd_config})
        if [ "${gui_attr}" != "" ]; then
            sed -i 's/PCSD_DISABLE_GUI=false/PCSD_DISABLE_GUI=true/g' ${pcsd_config}
        else
            echo "PCSD_DISABLE_GUI=true" >> ${pcsd_config}
        fi
     fi
}

validate_product() {
    valid_products=("${PRODUCT_CYBER_AI}" "${PRODUCT_ENTERPRISE}" "${PRODUCT_TELEMETRY}" "${PRODUCT_UFM_CYBER_AI}")

    if [[ ! " ${valid_products[*]} " =~ " ${PRODUCT} " ]]; then
        echo "Invalid product name! Product must be either: cyber-ai, enterprise, or telemetry."
        exit 1
    fi
}

validate_drbd_location() {
    echo "validate_drbd_location"
    # Check if the directory exists
    if [ ! -d ${DRBD_LOCATION} ] ; then
        echo "Cannot find ${DRBD_LOCATION} drbd location."
        exit 1
    fi
}

validate_drbd_disk() {
    echo "validate_drbd_disk"
    # Check if the block file exists
    if [ ! -b ${DRBD_DISK} ] ; then
        echo "Cannot find ${DRBD_DISK} drbd disk."
        exit 1
    fi
}


handle_product_settings() {
    \cp ${SCRIPT_DIR}/conf/${PRODUCT}-ha.conf ${UFM_HA_CONFIG}

    if [[ ${PRODUCT} == ${PRODUCT_ENTERPRISE} ]]; then

        # Fix UFM files path in UFM HA Watcher
        sed -i "s|/opt/ufm/ufm-enterprise/files|${DRBD_LOCATION}|" ${UFM_WATCHER_FILE}

        # Clean old HA flags in UFM Files if they exist
        rm -f ${DRBD_LOCATION}/clean_ha
        rm -f ${DRBD_LOCATION}/config_ha
    fi

    # Replace variables inside HA configuration file if the variable exists
    sed -i "s|@@DRBD_LOCATION@@|${DRBD_LOCATION}|g" ${UFM_HA_CONFIG}
    sed -i "s|@@DRBD_DISK@@|${DRBD_DISK}|g" ${UFM_HA_CONFIG}
}

check_version() {
    local_ufm_ha_cluster_file=${LOCAL_BIN_DIR}/ufm_ha_cluster

    local ha_installed=false
    if [[ -f "$local_ufm_ha_cluster_file" ]]; then
        ha_installed=true
    fi

    if [[ ${ha_installed} = true ]]; then 
        installed_version=$(grep 'UFM_HA_VERSION=' ${local_ufm_ha_cluster_file} | awk -F= '{print $NF}')
        ha_version=$(grep 'UFM_HA_VERSION=' ${SCRIPT_DIR}/scripts/ufm_ha_cluster | awk -F= '{print $NF}')
    
        local same_ha_version=false
        if [[ $installed_version == $ha_version ]]; then
            same_ha_version=true
        fi

        if [[ ${same_ha_version} == true ]]; then
            echo "UFM HA ${ha_version} is already installed on this system."
            exit 0
        fi
    fi
}
