#!/bin/bash
SCRIPT_NAME="$(basename "$0")"
SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"
. ${SCRIPT_DIR}/.install_common.sh


usage() {
    cat << EOF
Usage: ${SCRIPT_NAME} [ --drbd-location <DIRECTORY> ] [ --drbd-disk <PARTITION> ] [ --product [ cyber-ai | enterprise | telemetry ] ]

The ${SCRIPT_NAME} installs UFM HA plugin for UFM products (cyber-ai, enterprise, or telemetry).

OPTIONS:
    -l | --drbd-location           Directory used by DRBD to mount and sync UFM Files (default is /opt/ufm_files).
    -d | --drbd-disk               Name of disk partition for DRBD, for example: /dev/sdd4.
    -p | --product                 Name of UFM product to run in HA mode. Must be either: cyber-ai, enterprise, or telemetry.
    -h | --help                    Show this message.

EOF
}

parse_options() {

    short_opts="l:d:p:h"
    long_opts="drbd-location:,drbd-disk:,product:,help"
    options=$(getopt -n ${SCRIPT_NAME} -o ${short_opts} --long ${long_opts} -- "$@")
    if [ $? -ne 0 ]; then
      usage
      exit 1
    fi

    eval set -- "$options"
    while true
    do
        case "$1" in
            -l | --drbd-location)
                DRBD_LOCATION=$2
                shift 2
                ;;
            -d | --drbd-disk)
                DRBD_DISK=$2
                shift 2
                ;;
            -p | --product)
                PRODUCT=$2
                shift 2
                ;;
            -h | --help)
                usage
                exit 0
                ;;
            --)
                shift
                break
                ;;
        esac
    done

    if [[ -z ${PRODUCT} ]]; then
        echo "Error: missing product name!"
        usage
        exit 1
    else
        validate_product
    fi

    # For now, drbd-disk and drbd-location are not required in Cyber AI
    cyberai_products=("${PRODUCT_CYBER_AI}" "${PRODUCT_UFM_CYBER_AI}")
    if [[ ! " ${cyberai_products[*]} " =~ " ${PRODUCT} " ]]; then
        if [[ -z ${DRBD_LOCATION} ]]; then
            echo "Error: missing drbd-location!"
            usage
            exit 1
        else 
            validate_drbd_location
        fi

        if [[ -z ${DRBD_DISK} ]]; then
            echo "Error: missing drbd-disk!"
            usage
            exit 1
        else 
            validate_drbd_disk
        fi
    fi
}

#######################################################################
#                Installation Main Logic                              #
#######################################################################
parse_options "$@"

# For Appliance Skip Check Prereqs stage.
if [[ ! -f "/etc/ufm-release" ]]; then
    check_pre_reqs
    if [[ $? -ne 0 ]]; then
        echo "- ERROR - Requirements check failed. Exiting Installation!"
        exit 1
    fi
fi

echo "====================================" > ${INSTALL_LOG}
date >> ${INSTALL_LOG}
log_msg "===================================="
log_msg " [*] Installing UFM HA for ${PRODUCT} [*]"
log_msg " ----------------------------------"

check_version

log_msg "[*] Preparing configuration files"
handle_product_settings

pushd ${SCRIPT_DIR}/scripts > /dev/null
trap cleanup_install EXIT

log_msg "[*] Disabling pcs UI"
disable_pcs_gui

log_msg "[*] Copying files"
copy_files

echo "Done. Please run: 'systemctl daemon-reload' to load new HA services"
echo "To See full installation logs please check: ${INSTALL_LOG}"
