#!/bin/bash

SCRIPT_NAME="$(basename "$0")"
SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"


usage() {
    cat << EOF
Usage: ${SCRIPT_NAME} [OPTIONS]

${SCRIPT_NAME} configures UFM-HA cluster on both HA nodes. Run this script ONLY once on the HA master node.

[*] NOTE: ${SCRIPT_NAME} requires SSH trust established between HA nodes.

OPTIONS:
    --cluster-password            HA cluster password
    --master-ip                   Master node IP address
    --standby-ip                  Standby node IP address
    --virtual-ip                  HA Cluster virtual IP address
    --no-vip                      Do not create virtual IP resource.
                                  Mutual exclusive with virtual-ip.
    -h | --help                   Show this message.

EOF
}

NO_VIP=false


parse_options() {
    short_opts="h"
    long_opts="cluster-password:,master-ip:,standby-ip:,virtual-ip:,help,no-vip"
    options=$(getopt -n ${SCRIPT_NAME} -o ${short_opts} --long ${long_opts} -- "$@")
    if [ $? -ne 0 ]; then
      usage
      exit 1
    fi

    eval set -- "$options"
    while true
    do
        case "$1" in
            --cluster-password)
                cluster_pwd=$2
                shift 2
                ;;
            --master-ip)
                master_ip=$2
                shift 2
                ;;
            --standby-ip)
                standby_ip=$2
                shift 2
                ;;
            --virtual-ip)
                virtual_ip=$2
                shift 2
                ;;
            --no-vip)
                NO_VIP=true
                shift 1
                ;;
            -h | --help)
                usage
                exit 0
                ;;
            --)
                shift
                break
                ;;
        esac
    done

    if [[ -z ${cluster_pwd} ]]; then
        echo "ERROR - Missing argument: cluster-password"
    fi

    if [[ -z ${master_ip} ]]; then
        echo "ERROR - Missing argument: master-ip"
    fi

    if [[ -z ${standby_ip} ]]; then
        echo "ERROR - Missing argument: standby-ip"
    fi

    if [[ ${NO_VIP} == true && -n "${virtual_ip}" ]]; then
        echo "ERROR - the options: --no-vip and --virtual-ip are mutual exclusive and you can use only one of them."
        exit 1
    fi

    if [[ ${NO_VIP} != true &&  -z ${virtual_ip} ]]; then
        echo "ERROR - Missing argument: virtual-ip"
    fi

}

configure_ha(){
    /usr/local/bin/ufm_ha_cluster set-password -p ${cluster_pwd}
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to set password on Master node" && return ${exit_code}

    ssh ${standby_ip} "/usr/local/bin/ufm_ha_cluster set-password -p ${cluster_pwd}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to set password on Standby node" && return ${exit_code}

    ssh ${standby_ip} "/usr/local/bin/ufm_ha_cluster config -r standby -e ${master_ip} -l ${standby_ip} -p ${cluster_pwd}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to configure HA cluster on Standby node" && return ${exit_code}

    config_cmd="/usr/local/bin/ufm_ha_cluster config -r master -e ${standby_ip} -l ${master_ip} -p ${cluster_pwd}"
    if [[ ${NO_VIP} == true ]]; then
        config_cmd="${config_cmd} --no-vip"
    else
        config_cmd="${config_cmd} -i ${virtual_ip}"
    fi
    ${config_cmd}
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to configure HA cluster on Master node" && return ${exit_code}
}

parse_options "$@"

if [[ ${NO_VIP} == true ]]; then
    vip_message="NO-VIP"
else
    vip_message="${virtual_ip}"
fi

echo "==================="
echo " Configure HA Nodes"
echo "==================="
echo "Master IP:                 ${master_ip}"
echo "Standby IP:                ${standby_ip}"
echo "Virtual IP:                ${vip_message}"
configure_ha
